#include <functional>
#include <future>
#include <memory>
#include <string>
#include <sstream>

#include <rclcpp/rclcpp.hpp>
#include <rclcpp_action/rclcpp_action.hpp>
#include <franka_cps_msgs/action/grasp_sequence.hpp>

#include <moveit/planning_scene/planning_scene.h>
#include <moveit/planning_scene_interface/planning_scene_interface.h>
#include <moveit/task_constructor/task.h>
#include <moveit/task_constructor/solvers.h>
#include <moveit/task_constructor/stages.h>
#if __has_include(<tf2_geometry_msgs/tf2_geometry_msgs.hpp>)
#include <tf2_geometry_msgs/tf2_geometry_msgs.hpp>
#else
#include <tf2_geometry_msgs/tf2_geometry_msgs.h>
#endif
#if __has_include(<tf2_eigen/tf2_eigen.hpp>)
#include <tf2_eigen/tf2_eigen.hpp>
#else
#include <tf2_eigen/tf2_eigen.h>
#endif

static const rclcpp::Logger LOGGER = rclcpp::get_logger("franka_grasp_interface");
namespace mtc = moveit::task_constructor;

class GraspInterfaceNode
{
    public:
        GraspInterfaceNode(const rclcpp::NodeOptions& options);
    
        rclcpp::node_interfaces::NodeBaseInterface::SharedPtr getNodeBaseInterface();
    
        void doTask();
    
        void setupPlanningScene();

        rclcpp_action::create_client<franka_cps_msgs::action::GraspSequence>(this, "grasp_interface/grasp");
    private:
        mtc::Task createTask();
        mtc::Task task_;
        rclcpp::Node::SharedPtr node_;
}


GrapsInterfaceNode::GrapsInterfaceNode(const rclccp::NodeOptions& options)
    : node_{ std::make_shared<rclcpp::Node>("grasp_interface_node", options)}
{
}

rclcpp::node_interfaces::NodeBaseInterface::SharedPtr GraspInterfaceNode::getNodeBaseInterface()
{
    return node_->get_node_base_interface();
}

int main(int argc, char** argv)
{
    rclcpp::init(argc, argv);

    rclcpp::shutdown();
    return 0;
}